'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _xml2js = require('xml2js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Query = function () {
  function Query(jobWatch) {
    _classCallCheck(this, Query);

    this.restrictedExecutionTime = null;
    this.restrictedResources = [];
    this.restrictedName = null;
    this.restrictedDestination = null;
    this.restrictedStates = [];
    this.restrictedUser = '$USER';

    this.jobWatch = jobWatch;
  }

  _createClass(Query, [{
    key: 'resource',
    value: function resource(attribute, operator) {
      var value = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

      this.restrictedResources.push({
        attribute: attribute,
        operator: operator,
        value: value
      });

      return this;
    }
  }, {
    key: 'executionTime',
    value: function executionTime(operator, time) {
      var value = time;
      if (!_moment2.default.isMoment(time)) {
        value = (0, _moment2.default)(value);
      }

      this.restrictedExecutionTime = {
        operator: operator,
        value: value
      };

      return this;
    }
  }, {
    key: 'name',
    value: function (_name) {
      function name(_x) {
        return _name.apply(this, arguments);
      }

      name.toString = function () {
        return _name.toString();
      };

      return name;
    }(function (value) {
      this.restrictedName = name;
      return this;
    })
  }, {
    key: 'destination',
    value: function destination(value) {
      this.restrictedDestination = value;
      return this;
    }
  }, {
    key: 'states',
    value: function states() {
      for (var _len = arguments.length, values = Array(_len), _key = 0; _key < _len; _key++) {
        values[_key] = arguments[_key];
      }

      this.restrictedStates = values;
      return this;
    }
  }, {
    key: 'user',
    value: function user() {
      var username = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '$USER';

      this.restrictedUser = username;
      return this;
    }
  }, {
    key: 'getCommand',
    value: function getCommand() {
      var command = 'qselect';

      if (this.restrictedUser) {
        command += ' -u ' + this.restrictedUser;
      }

      if (this.restrictedName) {
        command += ' -N ' + this.restrictedName;
      }

      if (this.restrictedStates.length > 0) {
        command += ' -s ' + this.restrictedStates.join('');
      }

      if (this.restrictedExecutionTime) {
        var value = this.restrictedExecutionTime.value.format('YYYYMMDDHHmm.ss');
        command += ' -a ' + formatCondition(this.restrictedExecutionTime.operator, value.trim());
      }

      if (this.restrictedResources.length > 0) {
        command += ' -l ' + this.restrictedResources.map(function (resource) {
          return resource.attribute.trim() + formatCondition(resource.operator, resource.value.trim());
        }).join(',');
      }

      return command;
    }
  }, {
    key: 'getIds',
    value: function getIds() {
      var command = this.getCommand();

      return this.jobWatch.execute(command).then(function (output) {
        return output.split('\n').map(function (id) {
          return id.trim();
        }).filter(function (id) {
          return id !== '';
        });
      });
    }
  }, {
    key: 'getDetails',
    value: function getDetails() {
      var _this = this;

      var xmlParser = new _xml2js.Parser({ explicitArray: false });

      return this.getIds().then(function (ids) {
        if (ids.length > 0) {
          return _this.jobWatch.execute('qstat -x ' + ids.join(' '));
        } else {
          return '';
        }
      }).then(function (data) {
        return new Promise(function (resolve, reject) {
          var xmlString = '<Jobs>' + data + '</Jobs>';
          xmlParser.parseString(xmlString, function (err, result) {
            if (err) {
              reject(err);
            } else {
              resolve(result.Jobs.Data.map(function (job) {
                return job.Job;
              }));
            }
          });
        });
      });
    }
  }]);

  return Query;
}();

exports.default = Query;


var conditions = {
  '=': '.eq.',
  '!=': '.ne.',
  '<>': '.ne.',
  '>=': '.ge.',
  '<=': '.le.',
  '>': '.gt.',
  '<': '.lt.'
};

function formatCondition(operator, value) {
  return conditions[operator] + value;
}